/*###############################################################
# Note: the program is written in GAUSS. GAUSS system 
#       Version 7.0.13 or higher by Aptech Systems is required
#       to run it. 
# Purpose: To compute the likelihood function in Eq (9) and use
#          maxlik function provided by GAUSS to maximize the 
#          likelihood and obtain the MLEs
#
# Input: simdat001.txt (the simulated data set generated
#        by "generateData.R" in folder /generateData
#
# Output: the MLEs of all parameters
#         the log-likelihood computed at MLEs
#         the information matrix
#
# Authors: Sheng Luo (Advisor: Ciprian Crainiceanu)
# Date:    03/25/2007
############################################################### */

library maxlik;
maxset;

/* true_beta1: the true beta parameter for P_{i1}
   true_beta2: the true beta parameter for P_{i2}
   true_beta3: the true beta parameter for P_{i3} */
true_beta1 = {-1.75, -0.5}; true_beta2 = {-0.84, -0.25}; true_theta = {0.59, -0.33};
nbeta1 = rows(true_beta1); nbeta2 = rows(true_beta2); ntheta = rows(true_theta);
/* myP: the total number of beta parameters */
myP = nbeta1+nbeta2+ntheta;
/* n_sig: the total of dispersion parameters, i.e. theta1~3 */
n_sig = 3; 
/* nnols_dat: the total number of columns of input data file "simdat001.txt" */
ncols_dat = 9;

/* change the working directory to the one with the simulated data file */
chdir /data/luoshe/gauss/simulationForBeta_indep/N5000/simdat;
/* load in the data set simdat001.txt */ 
load dat[] = simdat001.txt;
/* By default, the data set read in by GAUSS is in a column vector. 
   It needs to be reshaped using reshape command below */ 
nrows = rows(dat)/ncols_dat;
/* Change the data set format from column vector to data frame*/
dat = reshape(dat, nrows, ncols_dat);
/* id: the study id of each subject */
/* nvisits: the number of follow-up visits */
/* x1: the covariate vector */
id = dat[.,1]; nvisits = dat[.,2]; x1 = dat[.,3:4];
/* k1, s1, k2, s2, n3: the sufficient statistics of the smoking patterns. 
                       Details in Section 2.2*/
k1 = dat[.,5]; s1 = dat[.,6]; k2 = dat[.,7]; s2 = dat[.,8];
n3 = dat[.,ncols_dat];

/* theta0: the initial values of the parameters, in the following order,
           {beta1, beta2, beta3, theta1, theta2, theta3}*/
theta0 = {-1, -1, -1, -1, 1, -1, 5, 1, 1};
/* _mlalgr, _mlstep, _mlcovp, _max_MaxIters are some parameters needed by GAUSS*/ 
_mlalgr = 2; _mlstep = 1; _mlcovp = 3; _max_MaxIters = 300;
/* Use GAUSS function maxlik to maximize the likelihood and obtain the MLEs */
{x, f, g, cov, retcode} = maxlik(x1, 0, &loglik, theta0);
/* After the computation, x contains the MLs*/ 
call maxprt(x, f, g, cov, retcode);

/* Compute the log-likelihood below */
/* beta: the MLEs for beta parameters */
/* sigma: the MLES for the thetas */ 
beta = x[1:myP]; sigma = x[myP+1:myP+n_sig];
/* lik: the vector to store the log-likelihood of each subject */
i = 1;  lik = {};

do while i<=nobs;
  temp = integrand(beta,sigma,x1[i,.],k1[i],s1[i],k2[i],s2[i],n3[i]);
  lik = lik|temp;
  i = i+1;
endo;
/* myloglik: the sum of the log-likelihood contribution of each subject */
myloglik = sumc(lik);
print myloglik;

/* change the working dirctory to save MLEs and log-likelihood */
chdir /data/luoshe/gauss/simulationForBeta_indep/N5000/mle;
outwidth 256;
output file = mle001.txt  reset;
print myloglik|x;
output off;

/* theta0: the MLEs */
/* delta0: the step length used in central difference scheme */
theta0 = x; delta0 = 1e-6;

/* infor: the information matrix */
infor = information(theta0, delta0, nrows);

/* change the working directory to store the information matrix */
chdir /data/luoshe/gauss/simulationForBeta_indep/N5000/inforMat;
outwidth 256;
output file = inforMat001.txt  reset;
print infor;
output off;

/* information: a function to compute the information matrix */
proc information(theta, delta, nsubjects);
  local theta_del_f, theta_del_b, lik_del_f, lik_del_b;
  local beta_del_f, beta_del_b, sigma_del_f, sigma_del_b;
  local lik_temp_f, lik_temp_b, temp_del_f, temp_del_b;
  local score, infor;
  local n_theta, i, j, temp;

/* n_theta: number of parameter, 9 in this case */
  n_theta = rows(theta);
/* theta_del_f: 9X9 matrix. In each row, add delta to one parameter estimate, 
   and other parameter estimates remain the same */
  theta_del_f = (theta[1]+delta)|theta[2:n_theta];
/* theta_del_b: 9X9 matrix. In each row, subtract delta to one parameter estimate, 
   and other parameter estimates remain the same */
  theta_del_b = (theta[1]-delta)|theta[2:n_theta];
  for i (2, (n_theta-1), 1);
    theta_del_f = theta_del_f~(theta[1:(i-1)]|(theta[i]+delta)|theta[(i+1):n_theta]);
    theta_del_b = theta_del_b~(theta[1:(i-1)]|(theta[i]-delta)|theta[(i+1):n_theta]);
  endfor;
  theta_del_f = theta_del_f~(theta[1:(n_theta-1)]|(theta[n_theta]+delta));
  theta_del_b = theta_del_b~(theta[1:(n_theta-1)]|(theta[n_theta]-delta));
  theta_del_f = theta_del_f';   theta_del_b = theta_del_b';

/* lik_del_f: NX9 matrix, each row stands for a subject, each col stands 
   for the log-likelihood under specified parameters in theta_del_f 
   for this subject, where N is the number of subjects */
/* lik_del_b: the same as lik_del_f except using theta_del_b */
  lik_del_f = {}; lik_del_b = {};
  for j (1, rows(theta_del_f), 1);
    beta_del_f = theta_del_f[j,1:myP]'; sigma_del_f = theta_del_f[j,myP+1:myP+n_sig]'; 
    beta_del_b = theta_del_b[j,1:myP]'; sigma_del_b = theta_del_b[j,myP+1:myP+n_sig]'; 
    lik_temp_f = {};  lik_temp_b = {}; 
    for i (1, nsubjects, 1);
      temp_del_f = integrand(beta_del_f,sigma_del_f,x1[i,.],k1[i],s1[i],k2[i],s2[i],n3[i]);
      temp_del_b = integrand(beta_del_b,sigma_del_b,x1[i,.],k1[i],s1[i],k2[i],s2[i],n3[i]);
      lik_temp_f = lik_temp_f|temp_del_f;
      lik_temp_b = lik_temp_b|temp_del_b;
    endfor;
    lik_del_f = lik_del_f~lik_temp_f; 
    lik_del_b = lik_del_b~lik_temp_b;
  endfor;  

/* score: 10,000X9 stores the score(first derivative) of each subject (10,000)
   for each parameter (9) */
  score = {};
  for i (1, nsubjects, 1);
    temp = {};
    for j (1, n_theta, 1);
/* using central difference scheme the approximate the first derivative */
      temp = temp~(lik_del_f[i,j]-lik_del_b[i,j])/(2*delta);
    endfor;
    score = score | temp;
  endfor;

/* infor: information matrix */
  infor = zeros(n_theta,n_theta);
  for i (1, nsubjects, 1);
    infor = infor + score[i,.]'*score[i,.];    
  endfor;
  infor = infor / nsubjects;
  retp(infor);
endp;

/* integrand: a fuction to compute the log-likelihood of one subject. */
/* beta: the current value vector of beta parameter 
   sigma: the currect value vector of the thetas
   tempX: the covariate vector of a subject 
   temp_k1, temp_s1, temp_k2, temp_s2, temp_n3: the sufficient stat of a subject */

proc integrand(beta,sigma,tempX,temp_k1,temp_s1,temp_k2,temp_s2,temp_n3);
  local mu_i1, mu_i2, mu_i3;
  local temp, term1, term2;
  local beta1, beta2, beta_3, theta1, theta2, theta3;
  local a1, a2, a3, b1, b2, b3;
  local liktemp, exponent;

/* beta1~3: the current value of beta parameters for P_{ij} for j=1,2,3 */ 
  beta1 = beta[1:nbeta1]; beta2 = beta[nbeta1+1:nbeta1+nbeta2];
  beta_3 = beta[nbeta1+nbeta2+1:myP];
/* theta1~3: the current values of the dispersion parameters */ 
  theta1 = sigma[1]; theta2 = sigma[2];  theta3 = sigma[3];

/* mu_i1,mu_i2,mu_i3: the mean of P_{ij}, see Eq (6) */
  mu_i1 = 1-exp(-exp(tempX*beta1));
  mu_i2 = 1-exp(-exp(tempX*beta2));
  exponent = exp(tempX*beta_3); mu_i3 = exponent/(1+exponent);

/* a_j: \theta_j * \mu_{ij} in Eq (9) 
   b_j: (1-\mu_{ij}) * \theta_j in Eq (9) for j=1,2,3 */ 
  a1 = theta1*mu_i1; b1 = theta1-a1;
  a2 = theta2*mu_i2; b2 = theta2-a2;
  a3 = theta3*mu_i3;  b3 = theta3-a3;

/* liktemp: the log-likelihood in Eq (9) */
  temp = logbetaFun(temp_k1+a1,temp_s1+b1)-logbetaFun(a1,b1)-logbetaFun(a2,b2)-logbetaFun(a3,b3);
  if temp_n3 == 0;
    term1 = logbetaFun(temp_k2+a2,temp_s2+b2);
    term2 = logbetaFun(a3,temp_k2+b3);
    liktemp = temp + term1 + term2;
  else;
    term1 = betaFun(temp_k2+a2,temp_s2+temp_n3+b2)*betaFun(a3,temp_k2+1+b3);
    term2 = betaFun(temp_k2+a2,temp_s2+b2)*betaFun(1+a3,temp_k2+b3); 
    liktemp = temp + ln(term1 + term2);
  endif;
  retp(liktemp);
endp;

/* loglik: a function to compute the log-likelood of all subjects*/
proc loglik(theta,z);
  local temp, x, beta, sigma, lik, i;

  beta = theta[1:myP]; sigma = theta[myP+1:myP+n_sig];  x = z[.,1:cols(z)];
  lik = {};

  if minc(sigma) > 0 ; /* This if sentense is to ensure that the thetas are positive */  
    for i (1, rows(z), 1);
      temp = integrand(beta,sigma,x[i,.],k1[i],s1[i],k2[i],s2[i],n3[i]);
      /* if the log-likelihood is too negative, it could a mistake */
      if temp < -1e8;
        lik = lik|temp;
        break;
      endif;
      lik = lik|temp;
    endfor;
    temp = sumc(lik);
  else;
    temp  = -1e20;
  endif;
  retp(temp);
endp;

/* betaFun: a function to compute the beta function B(a,b) */
proc betaFun(a, b);
  local delta;
  delta = 1e-6;
  if a<delta;
    a = delta;
  endif;
  if b<delta;
    b = delta;
  endif;
  retp(gamma(a)*gamma(b)/gamma(a+b));
endp;

/* logbetaFun: a function to compute the log of beta function B(a,b) */
proc logbetaFun(a, b);
  local delta;
  delta = 1e-6;
  if a<delta;
    a = delta;
  endif;
  if b<delta;
    b = delta;
  endif;
  retp(ln(gamma(a))+ln(gamma(b))-ln(gamma(a+b)));
endp;