################################################################
# Purpose: To generate the data set with structure 
#          similar to the original ATBC data set. 
#
# Input: nobs: number of study subjects
#        Beta1~3: the true beta parameter vector for
#                 transion probability P_{ij} for j=1,2,3
#        theta1~3:the dispersion parameter for
#                 transion probability P_{ij} for j=1,2,3
#
# Output: each subject's ID, smoking pattern, expressed in the
#         sufficient statistics in Section 2.2 of
#         Sheng's paper; covariate vector (contained intercept
#         and one binary covariate representing the presence
#         of baseline insomnia.
#
# Authors: Sheng Luo (Advisor: Ciprian Crainiceanu)
# Date:    03/25/2007
###############################################################

# inv.cloglog: Function to compute the inverse of
#              complementary-log-log link function
inv.cloglog <- function(x) {
  1-exp(-exp(x))
}

###############################################################
# main code starts from here
###############################################################
# groupdir: the directory to store the simulated data sets
groupdir <- "../simdat"
# nobs: number of study subjects
nobs <- 5000
# n.dataset: number of simulated data sets
n.dataset <- 100 

# myp: number of beta parameters for each P_{ij}
myp <- 2
# theta1~3: dispersion parameters
theta1 <- 10;  theta2 <- 3.20; theta3 <- 1.56

# Beta1: the beta parameter vector for P_{i1}
Beta1 <- matrix(c(-1.75, -0.5), nrow=myp)
# Beta2: the beta parameter vector for P_{i2}
Beta2 <- matrix(c(-0.84, -0.25), nrow=myp)
# Beta3: the beta parameter vector for P_{i3}
Beta3 <- matrix(c(0.59, -0.33), nrow=myp)

# c.mean: the mean number of follow-up visis
#         14.7 is the mean follow-up visit number in ATBC dataset
c.mean <- 14.7
# c.sd  : the standard deviation of the number of follow-up visits
c.sd <- 5.8

set.seed(1234)
# follow.up: the vector of the follow-up visit numbers
follow.up <- round(rnorm(nobs, mean = c.mean, sd = c.sd))
follow.up <- ifelse(follow.up < 1, 1, follow.up)


set.seed(13247)
# generate some random seeds for each simulated data set
my.seeds <- round(runif(n.dataset, 1, 10000))

###########################################################
# The main simulation starts from here. The for-loop below
# will generate 100 simulated data sets
# in the loop below, "0" is smoking and "1" is non-smoking
# SS: smoking status 
###########################################################
for (i.seed in 1:n.dataset){
  set.seed(my.seeds[i.seed])
  # X: the covariate vector;
  # About 20% subjects had baseline insomnia in ATBC study
  X <- cbind(rep(1,nobs), rbinom(nobs, 1, 0.2)) 

  # mu.i1~3: the mean vector for each P_{ij}; see Equation (6)  
  mu.i1 <- inv.cloglog(X %*% Beta1)
  mu.i2 <- inv.cloglog(X %*% Beta2)
  mu.i3 <- exp(X %*% Beta3) / (1 + exp(X %*% Beta3))
  
  # p1: the probability of making quit attempt
  # p2: the probability of relapsing
  # p3: the probability of being permanent quitter
  p1 <- rbeta(nobs, shape1 = theta1*mu.i1, shape2 = (1-mu.i1)*theta1)
  p2 <- rbeta(nobs, shape1 = theta2*mu.i2, shape2 = (1-mu.i2)*theta2)
  p3 <- rbeta(nobs, shape1 = theta3*mu.i3, shape2 = (1-mu.i3)*theta3)

  # smkPat: the vector of smoking patterns
  smkPat <- vector("list", nobs)

  ##########################################################
  # reminder: in the loop below, "0" is smoking
  #           and "1" is non-smoking. 
  #           SS: smoking status 
  ##########################################################
  for (i in 1:nobs) {
  # smk.temp: the character vector to store each subject's
  #           smoking pattern
    smk.temp <- NULL
  # bi.2: the variable to store the SS 2 intervals ago
  # set bi.2 to be 0 because everyone started out as a smoker
    bi.2 <- "0"  
  # bi.1: the variable to store the SS in last interval
  # simulate the smoking status in the 1st interval
    bi.1 <- as.character(rbinom(1, 1, p1[i]))
    smk.temp <- bi.1 # store the SS in the 1st interval
  # j:  the index of follow-up interval number
    j <- 2  # 2 stands for the 2nd follow-up visit
  # start the loop from the 2nd to last interval
    while(j <= follow.up[i]) {
  # if this is a quit attempt(switch from S to N)
  # reminder: quit attempt is defined as the non-smoking
  #           right after a smoking interval, e.g. SN
      if (bi.2 == "0" & bi.1 == "1") {
  #     simulate the indicator of cure status
  #     cured = 1: successfully quit; cured = 0: unsuccessfully quit
        cured <- rbinom(1, 1, p3[i])      
        if (cured == 1) {
  # if indicator shows "cured", fill in the remaining
  # intervals with 1(non-smoking)
          for (loop in j:follow.up[i])
            smk.temp <- paste(smk.temp, "1", sep = "")
          break # finish the simulation for the current subject
                # and start the simulation for next subject
        }
      }
      
      if (bi.1 == "0"){ # if subject smoked in last interval
        junk <- rbinom(1, 1, p1[i]) # simulate the SS in current interval
        if (junk == 1){ # junk = 1: subject switched from S to N
          temp <- as.character(1)
        } else { # junk == 0: subject kept smoking
          temp <- as.character(0)
        }
  # bi.1 = 1 here, meaning that subject didn't smoke in last interval
      } else { 
        junk <- rbinom(1, 1, p2[i]) # simulate the SS in current interval
        if (junk == 1) { # junk=1: subject relapse
          temp <- as.character(0)
        } else {    # junk=0: remain non-smoking
          temp <- as.character(1)
        }
      }
      bi.2 <- bi.1; bi.1 <- temp # update the SS in history
      # store the SS in current interval
      smk.temp <- paste(smk.temp, bi.1, sep = "") 
      j <- j + 1 # move on the next follow-up visit
    }
  # update the smoking pattern list for current subject
    smkPat[[i]] <- smk.temp 
  }
  smkPat <-  as.character(smkPat)
  # nVisits: number of follow-up visit for each subject
  nVisits <- nchar(smkPat)

  ##################################################################################
  # the code below is to generate the sufficient statistics of each smoking pattern
  ##################################################################################
  # find the patterns of at least 2 trailing "1"
  temp <- regexpr("1{2,}$", smkPat) 

  # truncate the smoking pattern into 2 sub-strings
  # i.e. smkPat.before and smkPat.after
  # smkPat.before: the string before trailing 1's - 1
  smkPat.before <- substring(smkPat, 1, as.integer(temp))
  # if no match, then smkPat.before should be the original string
  smkPat.before <- ifelse(as.integer(temp) < 0, smkPat, smkPat.before)

  # smkPat.after: the trailing 1's - 1, if it exists
  smkPat.after <- substring(smkPat, as.integer(temp) + 1)
  # if no match, then smkPat.after should be empty string
  smkPat.after <- ifelse(as.integer(temp) < 0, "", smkPat.after)

  #  in the loop below, "0" is smoking and "1" is non-smoking
  #  SS: smoking status 

  # K1: the total number of quit attempts
  # S1: the total number of smoking intervals
  #     excluding the relapsing intervals
  # K2: the total number of relapses 
  # S2: the total number of non-smoking intervals
  #     (excluding the quit attempts) in the
  #     type II geometric process with observed relapses
  # n3: the number of non-smoking intervals between the final
  #     quit attempt and censoring in the last observed interval
  #     is non-smoking and not a quit attempt
  K1 <- K2 <- S1 <- S2 <- n3 <- NULL
  for (i in 1:nobs) {
  # initialize some temporary variables for K1, K2, S1, S2 and n3
    temp.k1 <- temp.k2 <- temp.s1 <- temp.s2 <- temp.n3 <- 0

  # s1Leng: the length of first sub-string
    s1Leng <- nchar(smkPat.before[i])
    bi.2 <- 0 # every subject started out as smoker
    bi.1 <- as.integer(substr(smkPat.before[i],1,1)) # push out the 1st ss
    if (bi.1 == 1){ # if nonsmoke in the 1st interval
      temp.k1 <- temp.k1 + 1  # increment the number of quit attempts
    } else {
      temp.s1 <- temp.s1 + 1  # increment the number of smoking intervals
    }
  # smk.temp: the substring without the ss that is already read
    smk.temp <- substr(smkPat.before[i], 2, s1Leng)
  # li: the index for the first substring
    li <- 2
    while(li <= s1Leng) { # loop from the 2nd ss to the last ss
      temp <- as.integer(substr(smk.temp,1,1)) # push out the next SS
      if (bi.1 == 0){ # if subject smoked in last interval
        if (temp == 0){ # if the subject smoked in current interval
          temp.s1 <- temp.s1 + 1 # increment the number of smoking intervals
        } else {
          temp.k1 <- temp.k1 + 1 # increment the number of quit attempts
        }
      } else {
        if (bi.2 == 0){
          temp.k2 <- temp.k2 + 1 # increment the number of relapses
        }
        if (temp == 1){ # if the subject did NOT smoke in current inteval
          temp.s2 <- temp.s2 + 1 # increment the number of non-smoking intervals
        }
      }
      bi.2 <- bi.1;  bi.1 <- temp # update the ss in history
      # smk.temp: update smk.temp with the substring
      #           without the SS that is already read
      smk.temp <- substr(smk.temp, 2, nchar(smk.temp)) 
      li <- li + 1
    }
  # get the number of intervals in non-smoking state before censoring
    temp.n3 <- nchar(smkPat.after[i]) 
  
    K1 <- c(K1, temp.k1); K2 <- c(K2, temp.k2)
    S1 <- c(S1, temp.s1); S2 <- c(S2, temp.s2)
    n3 <- c(n3, temp.n3)
  }

  # subdat: the data set ready to write to a dat file
  subdat <- data.frame (ID = 1:nobs, nVisits = nVisits, X = X,
                        K1 = K1, S1 = S1, K2 = K2, S2 = S2, n3 = n3)

  temp <- i.seed
  if (i.seed < 10) temp <- paste("00", i.seed, sep="")
  if (i.seed >= 10 & i.seed < 100) temp <- paste("0", i.seed, sep="")
 # targetname: the name of data set
  targetname <- paste("simdat", temp, ".txt", sep="")
## create the file name with full path  
  wholefilename <- paste(groupdir, targetname, sep="/")
  write.table(subdat, wholefilename, sep = "\t",
              row.names = F, col.names = F)
}
